@@ -22,12 +22,11 @@ class Arpack(package.Package):
     self.archive        = 'arpack-ng-'+self.version+'.tar.gz'
     self.dirname        = 'arpack-ng-'+self.version
     self.supportssingle = True
-    self.supports64bint = True
     self.fortran        = True
     self.ProcessArgs(argdb)
 
   def Functions(self,petsc):
-    if petsc.mpiuni:
+    if petsc.mpiuni or petsc.msmpi:
       if petsc.scalar == 'real':
         if petsc.precision == 'single':
           functions = ['snaupd','sneupd','ssaupd','sseupd']
@@ -52,12 +51,12 @@ class Arpack(package.Package):
     return functions
 
 
-  def Check(self,conf,vars,petsc,archdir):
+  def Check(self,slepcconf,slepcvars,petsc,archdir):
     functions = self.Functions(petsc)
     if self.packagelibs:
       libs = [self.packagelibs]
     else:
-      if petsc.mpiuni:
+      if petsc.mpiuni or petsc.msmpi:
         libs = [['-larpack'],['-larpack_LINUX'],['-larpack_SUN4']]
       else:
         libs = [['-lparpack','-larpack'],['-lparpack_MPI','-larpack'],['-lparpack_MPI-LINUX','-larpack_LINUX'],['-lparpack_MPI-SUN4','-larpack_SUN4']]
@@ -66,21 +65,22 @@ class Arpack(package.Package):
       dirs = [os.path.join(self.packagedir,'lib'),self.packagedir]
     else:
       dirs = self.GenerateGuesses('Arpack',archdir)
-    self.FortranLib(conf,vars,dirs,libs,functions)
+    self.FortranLib(slepcconf,slepcvars,dirs,libs,functions)
 
 
-  def DownloadAndInstall(self,conf,vars,slepc,petsc,archdir,prefixdir):
+  def DownloadAndInstall(self,slepcconf,slepcvars,slepc,petsc,archdir,prefixdir):
     externdir = slepc.CreateDir(archdir,'externalpackages')
     builddir  = self.Download(externdir,slepc.downloaddir)
 
     # Check for autoreconf
-    result,output = self.RunCommand('autoreconf --help')
+    (result,output) = self.RunCommand('autoreconf --help')
     if result:
       self.log.Exit('--download-arpack requires that the command autoreconf is available on your PATH')
 
     # Build package
-    confopt = '--prefix='+prefixdir+' CC="'+petsc.cc+'" CFLAGS="'+petsc.cc_flags+'" F77="'+petsc.fc+'" FFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'" FC="'+petsc.fc+'" FCFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'"'
-    if not petsc.mpiuni:
+    extra_fcflags = ' -fallow-argument-mismatch' if petsc.isGfortran100plus() else ''
+    confopt = '--prefix='+prefixdir+' CC="'+petsc.cc+'" CFLAGS="'+petsc.cc_flags+'" F77="'+petsc.fc+'" FFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+extra_fcflags+'" FC="'+petsc.fc+'" FCFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+extra_fcflags+'" LIBS="'+petsc.blaslapack_lib+'"'
+    if not petsc.mpiuni and not petsc.msmpi:
       confopt = confopt+' --enable-mpi MPICC="'+petsc.cc+'" MPIF77="'+petsc.fc+'" MPIFC="'+petsc.fc+'"'
     if not petsc.buildsharedlib:
       confopt = confopt+' --disable-shared'
@@ -88,18 +88,17 @@ class Arpack(package.Package):
       if not petsc.blaslapackint64:
         self.log.Exit('To install ARPACK with 64-bit integers you also need a BLAS with 64-bit integers')
       confopt = confopt+' INTERFACE64=1'
-    result,output = self.RunCommand('cd '+builddir+'&& sh bootstrap && ./configure '+confopt+' && '+petsc.make+' && '+petsc.make+' install')
-    self.log.write(output)
+    (result,output) = self.RunCommand('cd '+builddir+'&& sh bootstrap && ./configure '+confopt+' && '+petsc.make+' && '+petsc.make+' install')
     if result:
       self.log.Exit('Installation of ARPACK failed')
 
     # Check build
     functions = self.Functions(petsc)
-    if petsc.mpiuni:
+    if petsc.mpiuni or petsc.msmpi:
       libs = [['-larpack']]
     else:
       libs = [['-lparpack','-larpack']]
     libdir = os.path.join(prefixdir,'lib')
     dirs = [libdir]
-    self.FortranLib(conf,vars,dirs,libs,functions)
+    self.FortranLib(slepcconf,slepcvars,dirs,libs,functions)
 
