@@ -3,18 +3,18 @@
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      13
-#define SLEPC_VERSION_SUBMINOR   1
+#define SLEPC_VERSION_MINOR      14
+#define SLEPC_VERSION_SUBMINOR   0
 #define SLEPC_VERSION_PATCH      0
-#define SLEPC_RELEASE_DATE       "March 31, 2020"
-#define SLEPC_VERSION_DATE       "Apr 12, 2020"
+#define SLEPC_RELEASE_DATE       "September 30, 2020"
+#define SLEPC_VERSION_DATE       "Sep 30, 2020"
 
 #if !defined (SLEPC_VERSION_GIT)
-#define SLEPC_VERSION_GIT        "v3.13.1"
+#define SLEPC_VERSION_GIT        "v3.14"
 #endif
 
 #if !defined(SLEPC_VERSION_DATE_GIT)
-#define SLEPC_VERSION_DATE_GIT   "2020-04-12 10:44:54 +0200"
+#define SLEPC_VERSION_DATE_GIT   "2020-09-30 17:08:23 +0200"
 #endif
 
 #define SLEPC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
