@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,8 +12,19 @@
 */
 
 #include <slepc/private/nepimpl.h>       /*I "slepcnep.h" I*/
+#include <slepc/private/bvimpl.h>        /*I "slepcbv.h" I*/
 #include <petscdraw.h>
 
+static PetscBool  cited = PETSC_FALSE;
+static const char citation[] =
+  "@Article{slepc-nep,\n"
+  "   author = \"C. Campos and J. E. Roman\",\n"
+  "   title = \"{NEP}: a module for the parallel solution of nonlinear eigenvalue problems in {SLEPc}\",\n"
+  "   journal = \"arXiv:1910.11712\",\n"
+  "   url = \"https://arxiv.org/abs/1910.11712\",\n"
+  "   year = \"2019\"\n"
+  "}\n";
+
 PetscErrorCode NEPComputeVectors(NEP nep)
 {
   PetscErrorCode ierr;
@@ -30,7 +41,7 @@ PetscErrorCode NEPComputeVectors(NEP nep)
 /*@
    NEPSolve - Solves the nonlinear eigensystem.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameter:
 .  nep - eigensolver context obtained from NEPCreate()
@@ -55,6 +66,7 @@ PetscErrorCode NEPSolve(NEP nep)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   if (nep->state>=NEP_STATE_SOLVED) PetscFunctionReturn(0);
+  ierr = PetscCitationsRegister(citation,&cited);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(NEP_Solve,nep,0,0,0);CHKERRQ(ierr);
 
   /* call setup */
@@ -70,10 +82,14 @@ PetscErrorCode NEPSolve(NEP nep)
   ierr = NEPViewFromOptions(nep,NULL,"-nep_view_pre");CHKERRQ(ierr);
   ierr = RGViewFromOptions(nep->rg,NULL,"-rg_view");CHKERRQ(ierr);
 
+  /* call solver */
   ierr = (*nep->ops->solve)(nep);CHKERRQ(ierr);
+  if (!nep->reason) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
   nep->state = NEP_STATE_SOLVED;
 
-  if (!nep->reason) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
+  /* Only the first nconv columns contain useful information */
+  ierr = BVSetActiveColumns(nep->V,0,nep->nconv);CHKERRQ(ierr);
+  if (nep->twosided) { ierr = BVSetActiveColumns(nep->W,0,nep->nconv);CHKERRQ(ierr); }
 
   if (nep->refine==NEP_REFINE_SIMPLE && nep->rits>0 && nep->nconv>0) {
     ierr = NEPComputeVectors(nep);CHKERRQ(ierr);
@@ -87,20 +103,23 @@ PetscErrorCode NEPSolve(NEP nep)
 
   /* various viewers */
   ierr = NEPViewFromOptions(nep,NULL,"-nep_view");CHKERRQ(ierr);
-  ierr = NEPReasonViewFromOptions(nep);CHKERRQ(ierr);
+  ierr = NEPConvergedReasonViewFromOptions(nep);CHKERRQ(ierr);
   ierr = NEPErrorViewFromOptions(nep);CHKERRQ(ierr);
   ierr = NEPValuesViewFromOptions(nep);CHKERRQ(ierr);
   ierr = NEPVectorsViewFromOptions(nep);CHKERRQ(ierr);
 
   /* Remove the initial subspace */
   nep->nini = 0;
+
+  /* Reset resolvent information */
+  ierr = MatDestroy(&nep->resolvent);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /*@
    NEPProjectOperator - Computes the projection of the nonlinear operator.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
@@ -145,7 +164,7 @@ PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1)
 /*@
    NEPApplyFunction - Applies the nonlinear function T(lambda) to a given vector.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep    - the nonlinear eigensolver context
@@ -167,7 +186,7 @@ PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1)
 
    Level: developer
 
-.seealso: NEPSetSplitOperator(), NEPComputeFunction()
+.seealso: NEPSetSplitOperator(), NEPComputeFunction(), NEPApplyAdjoint()
 @*/
 PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat A,Mat B)
 {
@@ -198,10 +217,62 @@ PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
   PetscFunctionReturn(0);
 }
 
+/*@
+   NEPApplyAdjoint - Applies the adjoint nonlinear function T(lambda)^* to a given vector.
+
+   Collective on nep
+
+   Input Parameters:
++  nep    - the nonlinear eigensolver context
+.  lambda - scalar argument
+.  x      - vector to be multiplied against
+-  v      - workspace vector (used only in the case of split form)
+
+   Output Parameters:
++  y   - result vector
+.  A   - Function matrix
+-  B   - optional preconditioning matrix
+
+   Level: developer
+
+.seealso: NEPSetSplitOperator(), NEPComputeFunction(), NEPApplyFunction()
+@*/
+PetscErrorCode NEPApplyAdjoint(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat A,Mat B)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscScalar    alpha;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidLogicalCollectiveScalar(nep,lambda,2);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,3);
+  if (v) PetscValidHeaderSpecific(v,VEC_CLASSID,4);
+  PetscValidHeaderSpecific(y,VEC_CLASSID,5);
+  if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,6);
+  if (B) PetscValidHeaderSpecific(B,MAT_CLASSID,7);
+
+  ierr = VecConjugate(x);CHKERRQ(ierr);
+  if (nep->fui==NEP_USER_INTERFACE_SPLIT) {
+    ierr = VecSet(y,0.0);CHKERRQ(ierr);
+    for (i=0;i<nep->nt;i++) {
+      ierr = FNEvaluateFunction(nep->f[i],lambda,&alpha);CHKERRQ(ierr);
+      ierr = MatMultTranspose(nep->A[i],x,v);CHKERRQ(ierr);
+      ierr = VecAXPY(y,alpha,v);CHKERRQ(ierr);
+    }
+  } else {
+    ierr = NEPComputeFunction(nep,lambda,A,B);CHKERRQ(ierr);
+    ierr = MatMultTranspose(A,x,y);CHKERRQ(ierr);
+  }
+  ierr = VecConjugate(x);CHKERRQ(ierr);
+  ierr = VecConjugate(y);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    NEPApplyJacobian - Applies the nonlinear Jacobian T'(lambda) to a given vector.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep    - the nonlinear eigensolver context
@@ -332,7 +403,9 @@ PetscErrorCode NEPGetConverged(NEP nep,PetscInt *nconv)
 .  NEP_CONVERGED_USER - converged due to a user-defined condition
 .  NEP_DIVERGED_ITS - required more than max_it iterations to reach convergence
 .  NEP_DIVERGED_BREAKDOWN - generic breakdown in method
--  NEP_DIVERGED_LINEAR_SOLVE - inner linear solve failed
+.  NEP_DIVERGED_LINEAR_SOLVE - inner linear solve failed
+-  NEP_DIVERGED_SUBSPACE_EXHAUSTED - run out of space for the basis in an
+   unrestarted solver
 
    Can only be called after the call to NEPSolve() is complete.
 
@@ -354,7 +427,7 @@ PetscErrorCode NEPGetConvergedReason(NEP nep,NEPConvergedReason *reason)
    NEPGetEigenpair - Gets the i-th solution of the eigenproblem as computed by
    NEPSolve(). The solution consists in both the eigenvalue and the eigenvector.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep - nonlinear eigensolver context
@@ -374,7 +447,8 @@ PetscErrorCode NEPGetConvergedReason(NEP nep,NEPConvergedReason *reason)
    If the eigenvalue is real, then eigi and Vi are set to zero. If PETSc is
    configured with complex scalars the eigenvalue is stored
    directly in eigr (eigi is set to zero) and the eigenvector in Vr (Vi is
-   set to zero). In both cases, the user can pass NULL in eigi and Vi.
+   set to zero). In any case, the user can pass NULL in Vr or Vi if one of
+   them is not required.
 
    The index i should be a value between 0 and nconv-1 (see NEPGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established
@@ -382,7 +456,7 @@ PetscErrorCode NEPGetConvergedReason(NEP nep,NEPConvergedReason *reason)
 
    Level: beginner
 
-.seealso: NEPSolve(), NEPGetConverged(), NEPSetWhichEigenpairs()
+.seealso: NEPSolve(), NEPGetConverged(), NEPSetWhichEigenpairs(), NEPGetLeftEigenvector()
 @*/
 PetscErrorCode NEPGetEigenpair(NEP nep,PetscInt i,PetscScalar *eigr,PetscScalar *eigi,Vec Vr,Vec Vi)
 {
@@ -410,24 +484,59 @@ PetscErrorCode NEPGetEigenpair(NEP nep,PetscInt i,PetscScalar *eigr,PetscScalar
 #endif
 
   /* eigenvector */
-#if defined(PETSC_USE_COMPLEX)
-  if (Vr) { ierr = BVCopyVec(nep->V,k,Vr);CHKERRQ(ierr); }
-  if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
-#else
-  if (nep->eigi[k]>0) { /* first value of conjugate pair */
-    if (Vr) { ierr = BVCopyVec(nep->V,k,Vr);CHKERRQ(ierr); }
-    if (Vi) { ierr = BVCopyVec(nep->V,k+1,Vi);CHKERRQ(ierr); }
-  } else if (nep->eigi[k]<0) { /* second value of conjugate pair */
-    if (Vr) { ierr = BVCopyVec(nep->V,k-1,Vr);CHKERRQ(ierr); }
-    if (Vi) {
-      ierr = BVCopyVec(nep->V,k,Vi);CHKERRQ(ierr);
-      ierr = VecScale(Vi,-1.0);CHKERRQ(ierr);
-    }
-  } else { /* real eigenvalue */
-    if (Vr) { ierr = BVCopyVec(nep->V,k,Vr);CHKERRQ(ierr); }
-    if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
-  }
-#endif
+  ierr = BV_GetEigenvector(nep->V,k,nep->eigi[k],Vr,Vi);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   NEPGetLeftEigenvector - Gets the i-th left eigenvector as computed by NEPSolve().
+
+   Logically Collective on nep
+
+   Input Parameters:
++  nep - eigensolver context
+-  i   - index of the solution
+
+   Output Parameters:
++  Wr   - real part of left eigenvector
+-  Wi   - imaginary part of left eigenvector
+
+   Notes:
+   The caller must provide valid Vec objects, i.e., they must be created
+   by the calling program with e.g. MatCreateVecs().
+
+   If the corresponding eigenvalue is real, then Wi is set to zero. If PETSc is
+   configured with complex scalars the eigenvector is stored directly in Wr
+   (Wi is set to zero). In any case, the user can pass NULL in Wr or Wi if one of
+   them is not required.
+
+   The index i should be a value between 0 and nconv-1 (see NEPGetConverged()).
+   Eigensolutions are indexed according to the ordering criterion established
+   with NEPSetWhichEigenpairs().
+
+   Left eigenvectors are available only if the twosided flag was set, see
+   NEPSetTwoSided().
+
+   Level: intermediate
+
+.seealso: NEPGetEigenpair(), NEPGetConverged(), NEPSetWhichEigenpairs(), NEPSetTwoSided()
+@*/
+PetscErrorCode NEPGetLeftEigenvector(NEP nep,PetscInt i,Vec Wr,Vec Wi)
+{
+  PetscErrorCode ierr;
+  PetscInt       k;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(nep,i,2);
+  if (Wr) { PetscValidHeaderSpecific(Wr,VEC_CLASSID,3); PetscCheckSameComm(nep,1,Wr,3); }
+  if (Wi) { PetscValidHeaderSpecific(Wi,VEC_CLASSID,4); PetscCheckSameComm(nep,1,Wi,4); }
+  NEPCheckSolved(nep,1);
+  if (!nep->twosided) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONGSTATE,"Must request left vectors with NEPSetTwoSided");
+  if (i<0 || i>=nep->nconv) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  ierr = NEPComputeVectors(nep);CHKERRQ(ierr);
+  k = nep->perm[i];
+  ierr = BV_GetEigenvector(nep->W,k,nep->eigi[k],Wr,Wi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -456,7 +565,7 @@ PetscErrorCode NEPGetErrorEstimate(NEP nep,PetscInt i,PetscReal *errest)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(errest,3);
+  PetscValidRealPointer(errest,3);
   NEPCheckSolved(nep,1);
   if (i<0 || i>=nep->nconv) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
   *errest = nep->errest[nep->perm[i]];
@@ -468,11 +577,12 @@ PetscErrorCode NEPGetErrorEstimate(NEP nep,PetscInt i,PetscReal *errest)
    associated with an eigenpair.
 
    Input Parameters:
+     adj    - whether the adjoint T^* must be used instead of T
      lambda - eigenvalue
      x      - eigenvector
      w      - array of work vectors (two vectors in split form, one vector otherwise)
 */
-PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscScalar lambda,Vec x,Vec *w,PetscReal *norm)
+PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscBool adj,PetscScalar lambda,Vec x,Vec *w,PetscReal *norm)
 {
   PetscErrorCode ierr;
   Vec            y,z=NULL;
@@ -480,7 +590,11 @@ PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscScalar lambda,Vec x,V
   PetscFunctionBegin;
   y = w[0];
   if (nep->fui==NEP_USER_INTERFACE_SPLIT) z = w[1];
-  ierr = NEPApplyFunction(nep,lambda,x,z,y,nep->function,nep->function_pre);CHKERRQ(ierr);
+  if (adj) {
+    ierr = NEPApplyAdjoint(nep,lambda,x,z,y,nep->function,nep->function_pre);CHKERRQ(ierr);
+  } else {
+    ierr = NEPApplyFunction(nep,lambda,x,z,y,nep->function,nep->function_pre);CHKERRQ(ierr);
+  }
   ierr = VecNorm(y,NORM_2,norm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -489,7 +603,7 @@ PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscScalar lambda,Vec x,V
    NEPComputeError - Computes the error (based on the residual norm) associated
    with the i-th computed eigenpair.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameter:
 +  nep  - the nonlinear eigensolver context
@@ -514,14 +628,14 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
   Vec            xr,xi=NULL;
   PetscInt       j,nwork,issplit=0;
   PetscScalar    kr,ki,s;
-  PetscReal      er,z=0.0;
+  PetscReal      er,z=0.0,errorl,nrm;
   PetscBool      flg;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,i,2);
   PetscValidLogicalCollectiveEnum(nep,type,3);
-  PetscValidPointer(error,4);
+  PetscValidRealPointer(error,4);
   NEPCheckSolved(nep,1);
 
   /* allocate work vectors */
@@ -545,9 +659,16 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
 #if !defined(PETSC_USE_COMPLEX)
   if (ki) SETERRQ(PetscObjectComm((PetscObject)nep),1,"Not implemented for complex eigenvalues with real scalars");
 #endif
-  ierr = NEPComputeResidualNorm_Private(nep,kr,xr,nep->work,error);CHKERRQ(ierr);
+  ierr = NEPComputeResidualNorm_Private(nep,PETSC_FALSE,kr,xr,nep->work,error);CHKERRQ(ierr);
   ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);
 
+  /* if two-sided, compute left residual norm and take the maximum */
+  if (nep->twosided) {
+    ierr = NEPGetLeftEigenvector(nep,i,xr,xi);CHKERRQ(ierr);
+    ierr = NEPComputeResidualNorm_Private(nep,PETSC_TRUE,kr,xr,nep->work,&errorl);CHKERRQ(ierr);
+    *error = PetscMax(*error,errorl);
+  }
+
   /* compute error */
   switch (type) {
     case NEP_ERROR_ABSOLUTE:
@@ -557,8 +678,11 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
       break;
     case NEP_ERROR_BACKWARD:
       if (nep->fui!=NEP_USER_INTERFACE_SPLIT) {
-        *error = 0.0;
-        ierr = PetscInfo(nep,"Backward error only available in split form\n");CHKERRQ(ierr);
+        ierr = NEPComputeFunction(nep,kr,nep->function,nep->function);CHKERRQ(ierr);
+        ierr = MatHasOperation(nep->function,MATOP_NORM,&flg);CHKERRQ(ierr);
+        if (!flg) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
+        ierr = MatNorm(nep->function,NORM_INFINITY,&nrm);CHKERRQ(ierr);
+        *error /= nrm*er;
         break;
       }
       /* initialization of matrix norms */
@@ -573,7 +697,7 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
         ierr = FNEvaluateFunction(nep->f[j],kr,&s);CHKERRQ(ierr);
         z = z + nep->nrma[j]*PetscAbsScalar(s);
       }
-      *error /= z;
+      *error /= z*er;
       break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid error type");
@@ -585,7 +709,7 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
    NEPComputeFunction - Computes the function matrix T(lambda) that has been
    set with NEPSetFunction().
 
-   Collective on NEP and Mat
+   Collective on nep
 
    Input Parameters:
 +  nep    - the NEP context
@@ -630,13 +754,6 @@ PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat A,Mat B)
     }
     if (A != B) SETERRQ(PetscObjectComm((PetscObject)nep),1,"Not implemented");
     break;
-  case NEP_USER_INTERFACE_DERIVATIVES:
-    ierr = PetscLogEventBegin(NEP_DerivativesEval,nep,A,B,0);CHKERRQ(ierr);
-    PetscStackPush("NEP user Derivatives function");
-    ierr = (*nep->computederivatives)(nep,lambda,0,A,nep->derivativesctx);CHKERRQ(ierr);
-    PetscStackPop;
-    ierr = PetscLogEventEnd(NEP_DerivativesEval,nep,A,B,0);CHKERRQ(ierr);
-    break;
   }
   PetscFunctionReturn(0);
 }
@@ -645,7 +762,7 @@ PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat A,Mat B)
    NEPComputeJacobian - Computes the Jacobian matrix T'(lambda) that has been
    set with NEPSetJacobian().
 
-   Collective on NEP and Mat
+   Collective on nep
 
    Input Parameters:
 +  nep    - the NEP context
@@ -687,13 +804,6 @@ PetscErrorCode NEPComputeJacobian(NEP nep,PetscScalar lambda,Mat A)
       ierr = MatAXPY(A,alpha,nep->A[i],nep->mstr);CHKERRQ(ierr);
     }
     break;
-  case NEP_USER_INTERFACE_DERIVATIVES:
-    ierr = PetscLogEventBegin(NEP_DerivativesEval,nep,A,0,0);CHKERRQ(ierr);
-    PetscStackPush("NEP user Derivatives function");
-    ierr = (*nep->computederivatives)(nep,lambda,1,A,nep->derivativesctx);CHKERRQ(ierr);
-    PetscStackPop;
-    ierr = PetscLogEventEnd(NEP_DerivativesEval,nep,A,0,0);CHKERRQ(ierr);
-    break;
   }
   PetscFunctionReturn(0);
 }
